@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-19  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -40,8 +40,8 @@ from photini.imagelist import ImageList
 from photini.loggerwindow import LoggerWindow
 from photini.opencage import OpenCage
 from photini.pyqt import (
-    catch_all, Qt, QtCore, QtGui, QNetworkProxy, QtWidgets, qt_version_info,
-    using_qtwebengine, width_for_text)
+    catch_all, Qt, QtCore, QtGui, QNetworkProxy, QtSlot, QtWidgets, qt_version,
+    qt_version_info, width_for_text)
 from photini.spelling import SpellCheck, spelling_version
 
 try:
@@ -54,6 +54,10 @@ except ImportError as ex:
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 
+if six.PY2:
+    logger.warning('Photini is currently using Python 2.'
+                   ' Support for Python 2 will be withdrawn in a future release.')
+
 
 class QTabBar(QtWidgets.QTabBar):
     def tabSizeHint(self, index):
@@ -80,7 +84,7 @@ class ConfigStore(BaseConfigStore, QtCore.QObject):
         super(ConfigStore, self).remove_section(section)
         self.timer.start()
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def save(self):
         super(ConfigStore, self).save()
@@ -93,7 +97,7 @@ class MainWindow(QtWidgets.QMainWindow):
             'MenuBar', "Photini photo metadata editor"))
         pixmap = QtGui.QPixmap()
         pixmap.loadFromData(pkg_resources.resource_string(
-            'photini', 'data/icons/48/photini.png'))
+            'photini', 'data/icons/photini_48.png'))
         icon = QtGui.QIcon(pixmap)
         self.setWindowIcon(icon)
         self.selection = list()
@@ -111,7 +115,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.app.config_store = ConfigStore('editor', parent=self)
         self.app.spell_check = SpellCheck(parent=self)
         self.app.open_cage = OpenCage(parent=self)
-        self.app.test_mode = options.test
+        self.app.options = options
         # restore size
         size = self.width(), self.height()
         self.resize(*eval(
@@ -166,54 +170,42 @@ class MainWindow(QtWidgets.QMainWindow):
             self.tab_list.append(tab)
         # file menu
         file_menu = self.menuBar().addMenu(translate('MenuBar', 'File'))
-        open_action = QtWidgets.QAction(
-            translate('MenuBar', 'Open images'), self)
-        open_action.setShortcuts(QtGui.QKeySequence.Open)
-        open_action.triggered.connect(self.image_list.open_files)
-        file_menu.addAction(open_action)
-        self.save_action = QtWidgets.QAction(
-            translate('MenuBar', 'Save images with new data'), self)
+        action = file_menu.addAction(translate('MenuBar', 'Open files'))
+        action.setShortcuts(QtGui.QKeySequence.Open)
+        action.triggered.connect(self.image_list.open_files)
+        self.save_action = file_menu.addAction(
+            translate('MenuBar', 'Save changes'))
         self.save_action.setShortcuts(QtGui.QKeySequence.Save)
         self.save_action.setEnabled(False)
         self.save_action.triggered.connect(self.image_list.save_files)
-        file_menu.addAction(self.save_action)
-        self.close_action = QtWidgets.QAction(
-            translate('MenuBar', 'Close selected images'), self)
-        self.close_action.setEnabled(False)
-        self.close_action.triggered.connect(self.close_files)
-        file_menu.addAction(self.close_action)
-        close_all_action = QtWidgets.QAction(
-            translate('MenuBar', 'Close all images'), self)
-        close_all_action.triggered.connect(self.close_all_files)
-        file_menu.addAction(close_all_action)
+        action = file_menu.addAction(translate('MenuBar', 'Close all files'))
+        action.triggered.connect(self.image_list.close_all_files)
+        sep = file_menu.addAction(translate('MenuBar', 'Selected images'))
+        sep.setSeparator(True)
+        self.selected_actions = self.image_list.add_selected_actions(file_menu)
         if GpxImporter:
-            file_menu.addSeparator()
-            self.import_gpx_action = QtWidgets.QAction(
-                translate('MenuBar', 'Import GPX file'), self)
+            self.import_gpx_action = file_menu.addAction(
+                translate('MenuBar', 'Import GPX file'))
             self.import_gpx_action.triggered.connect(self.import_pgx_file)
-            file_menu.addAction(self.import_gpx_action)
         else:
             self.import_gpx_action = None
         file_menu.addSeparator()
-        quit_action = QtWidgets.QAction(translate('MenuBar', 'Quit'), self)
-        quit_action.setShortcuts(
+        action = file_menu.addAction(translate('MenuBar', 'Quit'))
+        action.setShortcuts(
             [QtGui.QKeySequence.Quit, QtGui.QKeySequence.Close])
-        quit_action.triggered.connect(
+        action.triggered.connect(
             QtWidgets.QApplication.instance().closeAllWindows)
-        file_menu.addAction(quit_action)
         # options menu
         options_menu = self.menuBar().addMenu(translate('MenuBar', 'Options'))
-        settings_action = QtWidgets.QAction(
-            translate('MenuBar', 'Settings'), self)
-        settings_action.triggered.connect(self.edit_settings)
-        options_menu.addAction(settings_action)
+        action = options_menu.addAction(translate('MenuBar', 'Settings'))
+        action.triggered.connect(self.edit_settings)
         options_menu.addSeparator()
         for tab in self.tab_list:
             if tab['class']:
                 name = tab['name'].replace('&', '')
             else:
                 name = tab['module']
-            tab['action'] = QtWidgets.QAction(name, self)
+            tab['action'] = options_menu.addAction(name)
             tab['action'].setCheckable(True)
             if tab['class']:
                 tab['action'].setChecked(eval(
@@ -221,19 +213,17 @@ class MainWindow(QtWidgets.QMainWindow):
             else:
                 tab['action'].setEnabled(False)
             tab['action'].triggered.connect(self.add_tabs)
-            options_menu.addAction(tab['action'])
         # spelling menu
         languages = self.app.spell_check.available_languages()
         spelling_menu = self.menuBar().addMenu(translate('MenuBar', 'Spelling'))
-        enable_action = QtWidgets.QAction(
-            translate('MenuBar', 'Enable spell check'), self)
-        enable_action.setEnabled(languages is not None)
-        enable_action.setCheckable(True)
-        enable_action.setChecked(self.app.spell_check.enabled)
-        enable_action.toggled.connect(self.app.spell_check.enable)
-        spelling_menu.addAction(enable_action)
-        language_menu = QtWidgets.QMenu(
-            translate('MenuBar', 'Choose language'), self)
+        action = spelling_menu.addAction(
+            translate('MenuBar', 'Enable spell check'))
+        action.setEnabled(languages is not None)
+        action.setCheckable(True)
+        action.setChecked(self.app.spell_check.enabled)
+        action.toggled.connect(self.app.spell_check.enable)
+        language_menu = spelling_menu.addMenu(
+            translate('MenuBar', 'Choose language'))
         language_menu.setEnabled(languages is not None)
         current_language = self.app.spell_check.current_language()
         if languages:
@@ -241,30 +231,24 @@ class MainWindow(QtWidgets.QMainWindow):
             for name, code in languages:
                 if name != code:
                     name = code + ': ' + name
-                language_action = QtWidgets.QAction(name, self)
-                language_action.setCheckable(True)
-                language_action.setChecked(code == current_language)
-                language_action.setData(code)
-                language_action.setActionGroup(language_group)
-                language_menu.addAction(language_action)
+                action = language_menu.addAction(name)
+                action.setCheckable(True)
+                action.setChecked(code == current_language)
+                action.setData(code)
+                action.setActionGroup(language_group)
             language_group.triggered.connect(self.set_language)
         else:
-            language_action = QtWidgets.QAction(
-                translate('MenuBar', 'No dictionary installed'), self)
-            language_action.setEnabled(False)
-            language_menu.addAction(language_action)
-        spelling_menu.addMenu(language_menu)
+            action = language_menu.addAction(
+                translate('MenuBar', 'No dictionary installed'))
+            action.setEnabled(False)
         # help menu
         help_menu = self.menuBar().addMenu(translate('MenuBar', 'Help'))
-        about_action = QtWidgets.QAction(
-            translate('MenuBar', 'About Photini'), self)
-        about_action.triggered.connect(self.about)
-        help_menu.addAction(about_action)
+        action = help_menu.addAction(translate('MenuBar', 'About Photini'))
+        action.triggered.connect(self.about)
         help_menu.addSeparator()
-        help_action = QtWidgets.QAction(
-            translate('MenuBar', 'Photini documentation'), self)
-        help_action.triggered.connect(self.open_docs)
-        help_menu.addAction(help_action)
+        action = help_menu.addAction(
+            translate('MenuBar', 'Photini documentation'))
+        action.triggered.connect(self.open_docs)
         # main application area
         self.central_widget = QtWidgets.QSplitter()
         self.central_widget.setOrientation(Qt.Vertical)
@@ -286,12 +270,12 @@ class MainWindow(QtWidgets.QMainWindow):
         if self.initial_files:
             QtCore.QTimer.singleShot(0, self.open_initial_files)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def open_initial_files(self):
         self.image_list.open_file_list(self.initial_files)
 
-    @QtCore.pyqtSlot(bool)
+    @QtSlot(bool)
     @catch_all
     def add_tabs(self, checked):
         was_blocked = self.tabs.blockSignals(True)
@@ -316,23 +300,13 @@ class MainWindow(QtWidgets.QMainWindow):
             self.tabs.setCurrentWidget(current)
         self.new_tab(-1)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def open_docs(self):
         QtGui.QDesktopServices.openUrl(QtCore.QUrl(
             'http://photini.readthedocs.io/'))
 
-    @QtCore.pyqtSlot()
-    @catch_all
-    def close_files(self):
-        self.image_list.close_files(False)
-
-    @QtCore.pyqtSlot()
-    @catch_all
-    def close_all_files(self):
-        self.image_list.close_files(True)
-
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def import_pgx_file(self):
         importer = GpxImporter()
@@ -347,22 +321,22 @@ class MainWindow(QtWidgets.QMainWindow):
         self.image_list.unsaved_files_dialog(all_files=True, with_cancel=False)
         super(MainWindow, self).closeEvent(event)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def edit_settings(self):
         dialog = EditSettings(self)
         dialog.exec_()
         self.tabs.currentWidget().refresh()
 
-    @QtCore.pyqtSlot(QtWidgets.QAction)
+    @QtSlot(QtWidgets.QAction)
     @catch_all
     def set_language(self, action):
         self.app.spell_check.set_language(action.data())
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def about(self):
-        text = translate('MenuBar', """
+        text = """
 <table width="100%"><tr>
 <td align="center" width="70%">
 <h1>Photini</h1>
@@ -373,29 +347,38 @@ class MainWindow(QtWidgets.QMainWindow):
 </tr></table>
 <p>&copy; Jim Easterbrook <a href="mailto:jim@jim-easterbrook.me.uk">
 jim@jim-easterbrook.me.uk</a><br /><br />
-An easy to use digital photograph metadata editor.<br />
-Open source package available from
-<a href="https://github.com/jim-easterbrook/Photini">
-github.com/jim-easterbrook/Photini</a>.</p>
-<p>This program is released with a GNU General Public License. For
-details click the 'show details' button.</p>
-""").format(__version__, build,
-            pkg_resources.resource_filename(
-                'photini', 'data/icons/128/photini.png'))
+{3}<br />
+{4}</p>
+""".format(__version__, build,
+           pkg_resources.resource_filename(
+               'photini', 'data/icons/photini_128.png'),
+           translate('MenuBar',
+                     'An easy to use digital photograph metadata'
+                     ' (Exif, IPTC, XMP) editing application.'),
+           translate('MenuBar',
+                     'Open source package available from {}.').format(
+                         '<a href="https://github.com/jim-easterbrook/Photini">'
+                         'github.com/jim-easterbrook/Photini</a>'),
+           )
         dialog = QtWidgets.QMessageBox(self)
         dialog.setWindowTitle(translate('MenuBar', 'Photini: about'))
         dialog.setText(text)
         licence = pkg_resources.resource_string('photini', 'data/LICENSE.txt')
         dialog.setDetailedText(licence.decode('utf-8'))
+        dialog.setInformativeText(translate(
+            'MenuBar',
+            'This program is released with a GNU General Public'
+            ' License. For details click the "{}" button.').format(
+                dialog.buttons()[0].text()))
         dialog.exec_()
 
-    @QtCore.pyqtSlot(int, int)
+    @QtSlot(int, int)
     @catch_all
     def new_split(self, pos, index):
         self.app.config_store.set(
             'main_window', 'split', str(self.central_widget.sizes()))
 
-    @QtCore.pyqtSlot(int)
+    @QtSlot(int)
     @catch_all
     def new_tab(self, index):
         current = self.tabs.currentWidget()
@@ -404,17 +387,18 @@ details click the 'show details' button.</p>
             current.refresh()
             self.image_list.emit_selection()
 
-    @QtCore.pyqtSlot(list)
+    @QtSlot(list)
     @catch_all
     def new_selection(self, selection):
-        self.close_action.setEnabled(len(selection) > 0)
+        self.image_list.configure_selected_actions(self.selected_actions)
         if self.import_gpx_action:
             self.import_gpx_action.setEnabled(len(selection) > 0)
         self.tabs.currentWidget().new_selection(selection)
 
-    @QtCore.pyqtSlot(bool)
+    @QtSlot(bool)
     @catch_all
     def new_metadata(self, unsaved_data):
+        self.image_list.configure_selected_actions(self.selected_actions)
         self.save_action.setEnabled(unsaved_data)
 
     @catch_all
@@ -429,10 +413,10 @@ def main(argv=None):
     global app
     if argv:
         sys.argv = argv
-    # let PyQt handle its options (need at least one argument after options)
-    sys.argv.append('xxx')
+    # let Qt handle its options
     app = QtWidgets.QApplication(sys.argv)
-    del sys.argv[-1]
+    # get remaining argument list after Qt has swallowed its options
+    sys.argv = app.arguments()
     # install translations
     if qt_version_info < (5, 0):
         QtCore.QTextCodec.setCodecForTr(QtCore.QTextCodec.codecForName('utf-8'))
@@ -456,9 +440,7 @@ def main(argv=None):
     version = 'Photini ' + __version__ + ', build ' + build
     version += '\n  Python ' + sys.version
     version += '\n  ' + gi_version
-    version += '\n  PyQt {}, Qt {}, using {}'.format(
-        QtCore.PYQT_VERSION_STR, QtCore.QT_VERSION_STR,
-        ('QtWebKit', 'QtWebEngine')[using_qtwebengine])
+    version += '\n  ' + qt_version
     if spelling_version:
         version += '\n  ' + spelling_version
     if ffmpeg_version:
@@ -482,6 +464,10 @@ def main(argv=None):
         '-t', '--test', action='store_true',
         help=six.text_type(translate(
             'CLIHelp', 'test new features or API versions')))
+    parser.add_option(
+        '-u', '--utf_safe', action='store_true',
+        help=six.text_type(translate(
+            'CLIHelp', 'metadata is known to be ASCII or utf-8 encoded')))
     parser.add_option(
         '-v', '--verbose', action='count', default=0,
         help=six.text_type(translate(
