@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -36,7 +36,7 @@ import flickrapi
 from photini.configstore import key_store
 from photini.metadata import DateTime, LatLon, Location
 from photini.pyqt import (Busy, catch_all, MultiLineEdit, Qt, QtCore, QtGui,
-                          QtWidgets, SingleLineEdit)
+                          QtSignal, QtSlot, QtWidgets, SingleLineEdit)
 from photini.uploader import PhotiniUploader, UploaderSession
 
 logger = logging.getLogger(__name__)
@@ -54,7 +54,7 @@ class FlickrSession(UploaderSession):
         super(FlickrSession, self).__init__(*arg, **kwds)
         self.api = None
 
-    def connect(self):
+    def open_connection(self):
         api_key    = key_store.get('flickr', 'api_key')
         api_secret = key_store.get('flickr', 'api_secret')
         stored_token = self.get_password()
@@ -76,7 +76,7 @@ class FlickrSession(UploaderSession):
             return None
         return False
 
-    def disconnect(self):
+    def close_connection(self):
         self.connection_changed.emit(False)
         if self.api:
             # undocumented way to close Flickr connection cleanly
@@ -89,7 +89,7 @@ class FlickrSession(UploaderSession):
             return self.api.auth_url(perms='write')
         except Exception as ex:
             logger.error(str(ex))
-            self.disconnect()
+            self.close_connection()
         return ''
 
     def get_access_token(self, result):
@@ -98,7 +98,7 @@ class FlickrSession(UploaderSession):
             self.api.get_access_token(oauth_verifier)
         except Exception as ex:
             logger.error(str(ex))
-            self.disconnect()
+            self.close_connection()
             return
         token = self.api.token_cache.token
         self.set_password(token.token + '&' + token.token_secret)
@@ -114,7 +114,7 @@ class FlickrSession(UploaderSession):
                 return self.cached_data['user']
         except Exception as ex:
             logger.error(str(ex))
-            self.disconnect()
+            self.close_connection()
             return self.cached_data['user']
         user = rsp['oauth']['user']
         self.cached_data['user'] = user['fullname'], None
@@ -124,7 +124,7 @@ class FlickrSession(UploaderSession):
                 return self.cached_data['user']
         except Exception as ex:
             logger.error(str(ex))
-            self.disconnect()
+            self.close_connection()
             return self.cached_data['user']
         person = rsp['person']
         if person['iconserver'] != '0':
@@ -147,7 +147,7 @@ class FlickrSession(UploaderSession):
             sets = self.api.photosets.getList()
         except Exception as ex:
             logger.error(str(ex))
-            self.disconnect()
+            self.close_connection()
             return self.cached_data['sets']
         for item in sets['photosets']['photoset']:
             self.cached_data['sets'].append((
@@ -160,7 +160,7 @@ class FlickrSession(UploaderSession):
             rsp = self.api.photos.getInfo(photo_id=photo_id)
         except Exception as ex:
             logger.error(str(ex))
-            self.disconnect()
+            self.close_connection()
             return None
         if rsp['stat'] != 'ok':
             return None
@@ -180,7 +180,7 @@ class FlickrSession(UploaderSession):
                         return
                 except Exception as ex:
                     logger.error(str(ex))
-                    self.disconnect()
+                    self.close_connection()
                     return
             for photo in rsp['photos']['photo']:
                 yield photo
@@ -299,8 +299,8 @@ class FlickrSession(UploaderSession):
 
 
 class FlickrUploadConfig(QtWidgets.QWidget):
-    new_set = QtCore.pyqtSignal()
-    sync_metadata = QtCore.pyqtSignal()
+    new_set = QtSignal()
+    sync_metadata = QtSignal()
 
     def __init__(self, *arg, **kw):
         super(FlickrUploadConfig, self).__init__(*arg, **kw)
@@ -380,7 +380,7 @@ class FlickrUploadConfig(QtWidgets.QWidget):
         self.layout().addWidget(sets_group, 0, 2, 3, 1)
         self.layout().setColumnStretch(2, 1)
 
-    @QtCore.pyqtSlot(bool)
+    @QtSlot(bool)
     @catch_all
     def enable_ff(self, value):
         self.privacy['friends'].setEnabled(self.privacy['private'].isChecked())
@@ -531,17 +531,17 @@ class TabWidget(PhotiniUploader):
             if date_taken:
                 params['dates'] = {
                     'date_taken':
-                    date_taken.datetime.strftime('%Y-%m-%d %H:%M:%S')
+                    date_taken['datetime'].strftime('%Y-%m-%d %H:%M:%S')
                     }
-                if date_taken.precision <= 1:
+                if date_taken['precision'] <= 1:
                     params['dates']['date_taken_granularity'] = '6'
-                elif date_taken.precision <= 2:
+                elif date_taken['precision'] <= 2:
                     params['dates']['date_taken_granularity'] = '4'
             # location
             if image.metadata.latlong:
                 params['location'] = {
-                    'lat': '{:.6f}'.format(image.metadata.latlong.lat),
-                    'lon': '{:.6f}'.format(image.metadata.latlong.lon),
+                    'lat': '{:.6f}'.format(image.metadata.latlong['lat']),
+                    'lon': '{:.6f}'.format(image.metadata.latlong['lon']),
                     }
             else:
                 # clear any existing location
@@ -617,9 +617,9 @@ class TabWidget(PhotiniUploader):
         # get possible date range
         if not image.metadata.date_taken:
             return
-        precision = min(image.metadata.date_taken.precision, 6)
+        precision = min(image.metadata.date_taken['precision'], 6)
         min_taken_date = image.metadata.date_taken.truncate_datetime(
-            image.metadata.date_taken.datetime, precision)
+            image.metadata.date_taken['datetime'], precision)
         if precision >= 6:
             max_taken_date = min_taken_date + timedelta(seconds=1)
         elif precision >= 5:
@@ -651,7 +651,7 @@ class TabWidget(PhotiniUploader):
         for candidate in unknowns:
             if not candidate.metadata.date_taken:
                 continue
-            date_taken = candidate.metadata.date_taken.datetime
+            date_taken = candidate.metadata.date_taken['datetime']
             if date_taken < min_taken_date or date_taken > max_taken_date:
                 continue
             candidates.append(candidate)
@@ -734,10 +734,10 @@ class TabWidget(PhotiniUploader):
             if tag['raw'] == 'uploaded:by=photini':
                 continue
             if md.location_taken and tag['raw'] in (
-                    md.location_taken.country_code,
-                    md.location_taken.country_name,
-                    md.location_taken.province_state,
-                    md.location_taken.city):
+                    md.location_taken['country_code'],
+                    md.location_taken['country_name'],
+                    md.location_taken['province_state'],
+                    md.location_taken['city']):
                 continue
             tags.append(tag['raw'])
         md.keywords = md.keywords.merge(
@@ -781,7 +781,7 @@ class TabWidget(PhotiniUploader):
             else:
                 md.date_taken = date_taken
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def sync_metadata(self):
         # make list of known photo ids
@@ -812,7 +812,7 @@ class TabWidget(PhotiniUploader):
             for photo_id, image in photo_ids.items():
                 self._merge_metadata(photo_id, image)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def new_set(self):
         dialog = QtWidgets.QDialog(parent=self)
@@ -839,7 +839,7 @@ class TabWidget(PhotiniUploader):
         widget = self.upload_config.add_set(title, description, None, index=0)
         widget.setChecked(True)
 
-    @QtCore.pyqtSlot(list)
+    @QtSlot(list)
     @catch_all
     def new_selection(self, selection):
         super(TabWidget, self).new_selection(selection)
