@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -32,36 +32,28 @@ except ImportError:
     PIL = None
 
 from photini.ffmpeg import FFmpeg
-from photini.metadata import Metadata, MultiString
+from photini.metadata import Metadata
 from photini.pyqt import (
-    Busy, catch_all, image_types, Qt, QtCore, QtGui, QtWidgets, qt_version_info,
-    scale_font, set_symbol_font, video_types)
+    Busy, catch_all, image_types, Qt, QtCore, QtGui, QtSignal, QtSlot,
+    QtWidgets, qt_version_info, scale_font, set_symbol_font, video_types)
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 DRAG_MIMETYPE = 'application/x-photini-image'
 
 
-class TableWidget(QtWidgets.QTableWidget):
-    @catch_all
-    def sizeHint(self):
-        h_hdr = self.horizontalHeader()
-        v_hdr = self.verticalHeader()
-        return QtCore.QSize(h_hdr.length() + v_hdr.sizeHint().width() + 4,
-                            v_hdr.length() + h_hdr.sizeHint().height() + 4)
-
-
 class Image(QtWidgets.QFrame):
     def __init__(self, path, image_list, thumb_size=80, *arg, **kw):
         super(Image, self).__init__(*arg, **kw)
+        self.app = QtWidgets.QApplication.instance()
         self.path = path
         self.image_list = image_list
         self.name, ext = os.path.splitext(os.path.basename(self.path))
         self.selected = False
         self.thumb_size = thumb_size
         # read metadata
-        self.metadata = Metadata(self.path)
-        self.metadata.unsaved.connect(self.show_status)
+        self.metadata = Metadata(self.path, notify=self.show_status,
+                                 utf_safe=self.app.options.utf_safe)
         self.file_times = (os.path.getatime(self.path),
                            os.path.getmtime(self.path))
         # set file type
@@ -93,90 +85,13 @@ class Image(QtWidgets.QFrame):
         self.show_status(False)
         self._set_thumb_size(self.thumb_size)
 
-    @QtCore.pyqtSlot()
-    @catch_all
     def reload_metadata(self):
-        self.metadata = Metadata(self.path)
-        self.metadata.unsaved.connect(self.show_status)
+        self.metadata = Metadata(self.path, notify=self.show_status)
         self.show_status(False)
         self.load_thumbnail()
         self.image_list.emit_selection()
 
-    @QtCore.pyqtSlot()
-    @catch_all
-    def save_metadata(self):
-        self.image_list._save_files(images=[self])
-
-    @QtCore.pyqtSlot()
-    @catch_all
-    def diff_metadata(self):
-        dialog = QtWidgets.QDialog(parent=self)
-        dialog.setWindowTitle(translate('ImageList', 'Metadata differences'))
-        dialog.setLayout(QtWidgets.QVBoxLayout())
-        table = TableWidget()
-        table.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
-                            QtWidgets.QSizePolicy.Expanding)
-        table.setColumnCount(3)
-        table.setHorizontalHeaderLabels([translate('ImageList', 'new value'),
-                                         translate('ImageList', 'undo'),
-                                         translate('ImageList', 'old value')])
-        labels = []
-        row = 0
-        undo = {}
-        new_md = self.metadata
-        old_md = Metadata(self.path)
-        for key in ('title', 'description', 'keywords', 'rating',
-                    'copyright', 'creator',
-                    'date_taken', 'date_digitised', 'date_modified',
-                    'orientation',
-                    'lens_model', 'lens_make', 'lens_serial', 'lens_spec',
-                    'focal_length', 'focal_length_35', 'aperture',
-                    'latlong', 'altitude', 'location_taken', 'location_shown',
-                    'thumbnail'):
-            values = getattr(new_md, key), getattr(old_md, key)
-            if values[0] == values[1]:
-                continue
-            table.setRowCount(row + 1)
-            for n, value in enumerate(values):
-                if not value:
-                    value = ''
-                elif isinstance(value, MultiString):
-                    value = '\n'.join(value)
-                else:
-                    value = six.text_type(value)
-                item = QtWidgets.QTableWidgetItem(value)
-                table.setItem(row, n * 2, item)
-            undo[key] = QtWidgets.QTableWidgetItem()
-            undo[key].setFlags(undo[key].flags() | Qt.ItemIsUserCheckable)
-            undo[key].setCheckState(False)
-            table.setItem(row, 1, undo[key])
-            labels.append(key)
-            row += 1
-        table.setVerticalHeaderLabels(labels)
-        table.resizeColumnsToContents()
-        table.resizeRowsToContents()
-        dialog.layout().addWidget(table)
-        button_box = QtWidgets.QDialogButtonBox(
-            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
-        button_box.accepted.connect(dialog.accept)
-        button_box.rejected.connect(dialog.reject)
-        dialog.layout().addWidget(button_box)
-        if dialog.exec_() != QtWidgets.QDialog.Accepted:
-            return
-        changed = False
-        dirty = False
-        for key, widget in undo.items():
-            if widget.checkState() == Qt.Checked:
-                setattr(new_md, key, getattr(old_md, key))
-                changed = True
-            else:
-                dirty = True
-        if not dirty:
-            self.reload_metadata()
-        elif changed:
-            self.image_list.emit_selection()
-
-    def transform(self, pixmap, orientation, inverse=False):
+    def transform(self, pixmap, orientation):
         orientation = (orientation or 1) - 1
         if not orientation:
             return pixmap
@@ -190,31 +105,30 @@ class Image(QtWidgets.QFrame):
         if orientation & 0b100:
             # transpose horizontal & vertical
             transform = QtGui.QTransform(0, 1, 1, 0, 1, 1) * transform
-        if inverse:
-            transform = transform.transposed()
         return pixmap.transformed(transform)
 
-    @QtCore.pyqtSlot()
-    @catch_all
     def regenerate_thumbnail(self):
         # DCF spec says thumbnail must be 160 x 120, so other aspect
         # ratios are padded with black
-        with Busy():
-            # first try using FFmpeg to make thumbnail
-            data, fmt, w, h = self.make_thumb_ffmpeg()
-            if not data:
-                # use PIL or Qt
-                qt_im = self.get_qt_image()
-                if not qt_im:
-                    return
-                if PIL:
-                    data, fmt, w, h = self.make_thumb_PIL(qt_im)
-                else:
-                    data, fmt, w, h = self.make_thumb_Qt(qt_im)
-            # set thumbnail
-            self.metadata.thumbnail = data, fmt, w, h
-            # reload thumbnail
-            self.load_thumbnail()
+        # first try using FFmpeg to make thumbnail
+        data = self.make_thumb_ffmpeg()
+        if data:
+            self.metadata.thumbnail = {'data': data}
+            return True
+        # use PIL or Qt
+        qt_im = self.get_qt_image()
+        if not qt_im or qt_im.isNull():
+            return False
+        if PIL:
+            data = self.make_thumb_PIL(qt_im)
+            if data:
+                self.metadata.thumbnail = {'data': data}
+                return True
+        qt_im = self.make_thumb_Qt(qt_im)
+        if not qt_im or qt_im.isNull():
+            return False
+        self.metadata.thumbnail = {'image': qt_im}
+        return True
 
     def make_thumb_ffmpeg(self):
         # get input dimensions
@@ -224,7 +138,7 @@ class Image(QtWidgets.QFrame):
             logger.error(str(ex))
             dims = {}
         if not dims:
-            return None, 'JPEG', 0, 0
+            return None
         width = dims['width']
         height = dims['height']
         if 'duration' in dims:
@@ -243,29 +157,27 @@ class Image(QtWidgets.QFrame):
                 data = FFmpeg.make_thumbnail(self.path, w, h, skip, quality)
             except Exception as ex:
                 logger.error(str(ex))
-                return None, 'JPEG', 0, 0
+                return None
             if not data or len(data) < 50000:
                 break
             quality += 1
-        return data, 'JPEG', w, h
+        return data
 
     def get_qt_image(self):
-        qt_im = QtGui.QImage(self.path)
+        reader = QtGui.QImageReader(self.path)
+        reader.setAutoTransform(False)
+        qt_im = reader.read()
         if not qt_im or qt_im.isNull():
             logger.error('Cannot read %s image data from %s',
                          self.file_type, self.path)
             return None
-        # reorient if required
-        if self.file_type in ('image/x-canon-cr2', 'image/x-nikon-nef'):
-            qt_im = self.transform(
-                qt_im, self.metadata.orientation, inverse=True)
         w = qt_im.width()
         h = qt_im.height()
-        if max(w, h) > 6000:
-            # use Qt's scaling (not high quality) to pre-shrink very
-            # large images, to avoid PIL "DecompressionBombWarning"
-            return qt_im.scaled(
-                6000, 6000, Qt.KeepAspectRatio, Qt.SmoothTransformation)
+        if max(w, h) > 1000:
+            # use Qt's scaling (not high quality) to pre-shrink large
+            # images
+            qt_im = qt_im.scaled(
+                1000, 1000, Qt.KeepAspectRatio, Qt.SmoothTransformation)
             w = qt_im.width()
             h = qt_im.height()
         # pad image to 4:3 (or 3:4) aspect ratio
@@ -295,44 +207,33 @@ class Image(QtWidgets.QFrame):
             w, h = h, w
         # convert Qt image to PIL image
         buf = QtCore.QBuffer()
-        buf.open(QtCore.QIODevice.WriteOnly)
+        buf.open(buf.WriteOnly)
         qt_im.save(buf, 'PPM')
         data = BytesIO(buf.data().data())
-        pil_im = PIL.open(data)
+        try:
+            pil_im = PIL.open(data)
+        except Exception as ex:
+            logger.error(ex)
+            return None
         # scale PIL image
         pil_im.thumbnail((w, h), PIL.ANTIALIAS)
         # save image to memory
         data = BytesIO()
-        fmt = 'JPEG'
-        pil_im.save(data, fmt)
-        return (data.getvalue(), fmt) + pil_im.size
+        pil_im.save(data, 'JPEG')
+        return data.getvalue()
 
     def make_thumb_Qt(self, qt_im):
         w, h = 160, 120
         if qt_im.width() < qt_im.height():
             w, h = h, w
         # scale Qt image - not as good quality as PIL
-        qt_im = qt_im.scaled(
-            w, h, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
-        # save image to memory
-        buf = QtCore.QBuffer()
-        buf.open(QtCore.QIODevice.WriteOnly)
-        fmt = 'JPEG'
-        qt_im.save(buf, fmt)
-        return buf.data().data(), fmt, qt_im.width(), qt_im.height()
+        return qt_im.scaled(w, h, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
 
     @catch_all
     def contextMenuEvent(self, event):
         menu = QtWidgets.QMenu(self)
-        menu.addAction(translate('ImageList', 'Reload metadata'),
-                       self.reload_metadata)
-        menu.addAction(translate('ImageList', 'Save metadata'),
-                       self.save_metadata)
-        menu.addAction(translate('ImageList', 'View changes'),
-                       self.diff_metadata)
-        menu.addAction(translate('ImageList', 'Regenerate thumbnail'),
-                       self.regenerate_thumbnail)
-        action = menu.exec_(event.globalPos())
+        self.image_list.add_selected_actions(menu)
+        menu.exec_(event.globalPos())
 
     @catch_all
     def mousePressEvent(self, event):
@@ -376,10 +277,13 @@ class Image(QtWidgets.QFrame):
         else:
             icon = QtGui.QPixmap(src_w + margin, src_h + margin)
             icon.fill(Qt.transparent)
-            with QtGui.QPainter(icon) as paint:
+            try:
+                paint = QtGui.QPainter(icon)
                 for i in range(count):
                     paint.drawPixmap(
                         QtCore.QPoint(margin - (i * 4), i * 4), src_icon)
+            finally:
+                del paint
         drag.setPixmap(icon)
         if self.image_list.drag_hotspot:
             x, y = self.image_list.drag_hotspot
@@ -395,8 +299,6 @@ class Image(QtWidgets.QFrame):
     def mouseDoubleClickEvent(self, event):
         QtGui.QDesktopServices.openUrl(QtCore.QUrl.fromLocalFile(self.path))
 
-    @QtCore.pyqtSlot(bool)
-    @catch_all
     def show_status(self, changed):
         status = ''
         # set 'geotagged' status
@@ -426,13 +328,11 @@ class Image(QtWidgets.QFrame):
         self.load_thumbnail()
 
     def load_thumbnail(self):
-        pixmap = QtGui.QPixmap()
-        thumb = self.metadata.thumbnail
-        if thumb:
-            pixmap.loadFromData(thumb.data)
-        if pixmap.isNull():
+        image = self.metadata.thumbnail and self.metadata.thumbnail['image']
+        if not image:
             self.image.setText(translate('ImageList', 'No\nthumbnail\nin file'))
             return
+        pixmap = QtGui.QPixmap.fromImage(image)
         pixmap = self.transform(pixmap, self.metadata.orientation)
         self.image.setPixmap(
             pixmap.scaled(self.thumb_size, self.thumb_size,
@@ -450,7 +350,7 @@ class Image(QtWidgets.QFrame):
 
 
 class ScrollArea(QtWidgets.QScrollArea):
-    dropped_images = QtCore.pyqtSignal(list)
+    dropped_images = QtSignal(list)
 
     def __init__(self, parent=None):
         super(ScrollArea, self).__init__(parent)
@@ -600,10 +500,10 @@ class ThumbsLayout(QtWidgets.QLayout):
 
 
 class ImageList(QtWidgets.QWidget):
-    image_list_changed = QtCore.pyqtSignal()
-    new_metadata = QtCore.pyqtSignal(bool)
-    selection_changed = QtCore.pyqtSignal(list)
-    sort_order_changed = QtCore.pyqtSignal()
+    image_list_changed = QtSignal()
+    new_metadata = QtSignal(bool)
+    selection_changed = QtSignal(list)
+    sort_order_changed = QtSignal()
 
     def __init__(self, parent=None):
         super(ImageList, self).__init__(parent)
@@ -684,9 +584,9 @@ class ImageList(QtWidgets.QWidget):
             self.selection_anchor = None
             self.emit_selection()
 
-    @QtCore.pyqtSlot(bool)
+    @QtSlot(bool)
     @catch_all
-    def open_files(self, checked):
+    def open_files(self, checked=False):
         args = [
             self,
             self.tr('Open files'),
@@ -712,7 +612,7 @@ class ImageList(QtWidgets.QWidget):
             path_list = list(map(unquote, path_list))
         self.open_file_list(path_list)
 
-    @QtCore.pyqtSlot(list)
+    @QtSlot(list)
     @catch_all
     def open_file_list(self, path_list):
         with Busy():
@@ -732,7 +632,8 @@ class ImageList(QtWidgets.QWidget):
         self.show_thumbnail(image)
 
     def done_opening(self, path):
-        self.app.config_store.set('paths', 'images', os.path.dirname(path))
+        self.app.config_store.set(
+            'paths', 'images', os.path.dirname(os.path.abspath(path)))
         self._sort_thumbnails()
 
     def _date_key(self, image):
@@ -745,13 +646,13 @@ class ImageList(QtWidgets.QWidget):
             # use file date as last resort
             result = datetime.fromtimestamp(os.path.getmtime(image.path))
         else:
-            result = result.datetime
+            result = result['datetime']
         # convert result to string and append path so photos with same
         # time stamp get sorted consistently
         result = result.strftime('%Y%m%d%H%M%S%f') + image.path
         return result
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def _new_sort_order(self):
         self._sort_thumbnails()
@@ -782,6 +683,146 @@ class ImageList(QtWidgets.QWidget):
             self.scroll_area.ensureWidgetVisible(image)
             self.app.processEvents()
 
+    def add_selected_actions(self, menu):
+        actions = {}
+        actions['reload'] = menu.addAction('', self.reload_selected_metadata)
+        actions['save'] = menu.addAction(
+            translate('ImageList', 'Save changes'),
+            self.save_selected_metadata)
+        actions['diff'] = menu.addAction(
+            translate('ImageList', 'View changes'),
+            self.diff_selected_metadata)
+        actions['thumbs'] = menu.addAction(
+            '', self.regenerate_selected_thumbnails)
+        actions['close'] = menu.addAction('', self.close_selected_files)
+        self.configure_selected_actions(actions)
+        return actions
+
+    def configure_selected_actions(self, actions):
+        images = self.get_selected_images()
+        changed_images = any([x.metadata.changed() for x in images])
+        actions['reload'].setEnabled(bool(images))
+        actions['save'].setEnabled(changed_images)
+        actions['diff'].setEnabled(changed_images)
+        actions['thumbs'].setEnabled(bool(images))
+        actions['close'].setEnabled(bool(images))
+        actions['reload'].setText(
+            translate('ImageList', 'Reload file(s)', '', len(images)))
+        actions['thumbs'].setText(
+            translate('ImageList', 'Regenerate thumbnail(s)', '', len(images)))
+        actions['close'].setText(
+            translate('ImageList', 'Close file(s)', '', len(images)))
+
+    @QtSlot()
+    @catch_all
+    def reload_selected_metadata(self):
+        with Busy():
+            for image in self.get_selected_images():
+                image.reload_metadata()
+
+    @QtSlot()
+    @catch_all
+    def save_selected_metadata(self):
+        self._save_files(images=self.get_selected_images())
+
+    @QtSlot()
+    @catch_all
+    def diff_selected_metadata(self):
+        dialog = QtWidgets.QDialog(parent=self)
+        dialog.setLayout(QtWidgets.QVBoxLayout())
+        dialog.setFixedSize(min(800, self.window().width()),
+                            min(400, self.window().height()))
+        table = QtWidgets.QTableWidget()
+        table.setColumnCount(3)
+        table.setHorizontalHeaderLabels([translate('ImageList', 'new value'),
+                                         translate('ImageList', 'undo'),
+                                         translate('ImageList', 'old value')])
+        table.horizontalHeader().setSectionResizeMode(
+            0, QtWidgets.QHeaderView.Stretch)
+        table.horizontalHeader().setSectionResizeMode(
+            2, QtWidgets.QHeaderView.Stretch)
+        dialog.layout().addWidget(table)
+        button_box = QtWidgets.QDialogButtonBox(
+            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
+        button_box.accepted.connect(dialog.accept)
+        button_box.rejected.connect(dialog.reject)
+        dialog.layout().addWidget(button_box)
+        changed = False
+        position = None
+        for image in self.get_selected_images():
+            if not image.metadata.changed():
+                continue
+            dialog.setWindowTitle(translate(
+                'ImageList', 'Metadata differences: {}').format(image.name))
+            labels = []
+            row = 0
+            undo = {}
+            table.clearContents()
+            new_md = image.metadata
+            old_md = Metadata(image.path)
+            for key in ('title', 'description', 'keywords', 'rating',
+                        'copyright', 'creator',
+                        'date_taken', 'date_digitised', 'date_modified',
+                        'orientation', 'camera_model',
+                        'lens_model', 'lens_spec',
+                        'focal_length', 'focal_length_35', 'aperture',
+                        'latlong', 'altitude',
+                        'location_taken', 'location_shown',
+                        'thumbnail'):
+                values = getattr(new_md, key), getattr(old_md, key)
+                if values[0] == values[1]:
+                    continue
+                values = [six.text_type(x or '') for x in values]
+                table.setRowCount(row + 1)
+                for n, value in enumerate(values):
+                    item = QtWidgets.QTableWidgetItem(value)
+                    table.setItem(row, n * 2, item)
+                undo[key] = QtWidgets.QTableWidgetItem()
+                undo[key].setFlags(undo[key].flags() | Qt.ItemIsUserCheckable)
+                undo[key].setCheckState(Qt.Unchecked)
+                table.setItem(row, 1, undo[key])
+                labels.append(key)
+                row += 1
+            if not row:
+                continue
+            table.setVerticalHeaderLabels(labels)
+            table.resizeColumnsToContents()
+            table.resizeRowsToContents()
+            if position:
+                dialog.move(position)
+            if dialog.exec_() != QtWidgets.QDialog.Accepted:
+                return
+            position = dialog.pos()
+            undo_all = True
+            for key, widget in undo.items():
+                if widget.checkState() == Qt.Checked:
+                    setattr(new_md, key, getattr(old_md, key))
+                    changed = True
+                else:
+                    undo_all = False
+            if undo_all:
+                image.reload_metadata()
+        if changed:
+            self.emit_selection()
+
+    @QtSlot()
+    @catch_all
+    def regenerate_selected_thumbnails(self):
+        with Busy():
+            for image in self.get_selected_images():
+                if image.regenerate_thumbnail():
+                    image.load_thumbnail()
+
+    @QtSlot()
+    @catch_all
+    def close_selected_files(self):
+        self.close_files(False)
+
+    @QtSlot()
+    @catch_all
+    def close_all_files(self):
+        self.close_files(True)
+
     def close_files(self, all_files):
         if not self.unsaved_files_dialog(all_files=all_files):
             return
@@ -804,9 +845,9 @@ class ImageList(QtWidgets.QWidget):
             self.emit_selection()
         self.image_list_changed.emit()
 
-    @QtCore.pyqtSlot(bool)
+    @QtSlot(bool)
     @catch_all
-    def save_files(self, checked):
+    def save_files(self, checked=False):
         self._save_files(self.images)
 
     def _save_files(self, images=[]):
@@ -842,7 +883,7 @@ class ImageList(QtWidgets.QWidget):
                 break
         else:
             return True
-        dialog = QtWidgets.QMessageBox()
+        dialog = QtWidgets.QMessageBox(parent=self)
         dialog.setWindowTitle(self.tr('Photini: unsaved data'))
         dialog.setText(self.tr('<h3>Some images have unsaved metadata.</h3>'))
         dialog.setInformativeText(self.tr('Do you want to save your changes?'))
@@ -903,7 +944,7 @@ class ImageList(QtWidgets.QWidget):
             idx = 0
         self.select_image(self.images[idx], extend_selection=extend_selection)
 
-    @QtCore.pyqtSlot(int)
+    @QtSlot(int)
     @catch_all
     def _new_thumb_size(self, value):
         self.thumb_size = value * 20
