@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -16,6 +16,7 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
+from collections import deque
 from contextlib import contextmanager
 from datetime import datetime
 import logging
@@ -33,7 +34,8 @@ except ImportError:
 from photini.metadata import Metadata
 from photini.pyqt import (
     Busy, catch_all, image_types_lower, Qt, QtCore, QtGui,
-    QtWidgets, qt_version_info, StartStopButton, video_types_lower)
+    QtSignal, QtSlot, QtWidgets, qt_version_info, StartStopButton,
+    video_types_lower)
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
@@ -50,6 +52,8 @@ class FolderSource(object):
             return None
         file_list = []
         for root, dirs, files in os.walk(self.root):
+            # ignore special directories such as .thumbs
+            dirs[:] = [x for x in dirs if x[0] != '.']
             for name in files:
                 base, ext = os.path.splitext(name)
                 if ext.lower() in self.image_types:
@@ -66,14 +70,14 @@ class FolderSource(object):
                 # use file date as last resort
                 timestamp = datetime.fromtimestamp(os.path.getmtime(path))
             else:
-                timestamp = timestamp.datetime
-            if metadata._sc:
-                sc_path = metadata._sc._path
-            else:
-                sc_path = None
+                timestamp = timestamp['datetime']
+            sc_path = metadata.find_sidecar()
             name = os.path.basename(path)
+            camera = metadata.camera_model
+            if camera:
+                camera = camera['model']
             file_data[name] = {
-                'camera'    : metadata.camera_model,
+                'camera'    : camera,
                 'path'      : path,
                 'sc_path'   : sc_path,
                 'name'      : name,
@@ -178,32 +182,30 @@ class CameraSource(object):
 
 
 class FileCopier(QtCore.QObject):
-    output = QtCore.pyqtSignal(dict, six.text_type)
-
-    def __init__(self, source, copy_list, move, updating, *args, **kwds):
+    def __init__(self, source, copy_list, move, copier_result, *args, **kwds):
         super(FileCopier, self).__init__(*args, **kwds)
         self.source = source
         self.copy_list = copy_list
         self.move = move
-        self.updating = updating
+        self.copier_result = copier_result
         self.running = True
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def start(self):
         status = 'ok'
         try:
             for info in self.source.copy_files(self.copy_list, self.move):
-                # don't display image until previous one has been displayed
-                self.updating.lock()
-                self.output.emit(info, status)
-                self.updating.unlock()
+                self.copier_result.append((info, status))
+                # wait for image display to show previous image(s)
+                while self.running and len(self.copier_result) > 1:
+                    QtCore.QThread.yieldCurrentThread()
                 if not self.running:
                     break
         except Exception as ex:
             status = str(ex)
             logger.error(status)
-        self.output.emit({}, status)
+        self.copier_result.append(({}, status))
 
 
 def get_camera_list():
@@ -218,14 +220,14 @@ def get_camera_list():
 
 class NameMangler(QtCore.QObject):
     number_parser = re.compile(r'(\d+)')
-    new_example = QtCore.pyqtSignal(str)
+    new_example = QtSignal(str)
 
     def __init__(self, parent=None):
         super(NameMangler, self).__init__(parent)
         self.example = None
         self.format_string = None
 
-    @QtCore.pyqtSlot(str)
+    @QtSlot(str)
     @catch_all
     def new_format(self, format_string):
         self.format_string = format_string
@@ -276,11 +278,11 @@ class TabWidget(QtWidgets.QWidget):
 
     def __init__(self, image_list, parent=None):
         super(TabWidget, self).__init__(parent)
-        app = QtWidgets.QApplication.instance()
-        app.aboutToQuit.connect(self.shutdown)
-        if gp and app.test_mode:
+        self.app = QtWidgets.QApplication.instance()
+        self.app.aboutToQuit.connect(self.stop_copy)
+        if gp and self.app.options.test:
             self.gp_log = gp.check_result(gp.use_python_logging())
-        self.config_store = app.config_store
+        self.config_store = self.app.config_store
         self.image_list = image_list
         self.setLayout(QtWidgets.QGridLayout())
         form = QtWidgets.QFormLayout()
@@ -361,7 +363,7 @@ class TabWidget(QtWidgets.QWidget):
         self.refresh()
         self.list_files()
 
-    @QtCore.pyqtSlot(int)
+    @QtSlot(int)
     @catch_all
     def new_source(self, idx):
         self.source = None
@@ -404,7 +406,7 @@ class TabWidget(QtWidgets.QWidget):
         idx = self.source_selector.count() - (1 + len(folders))
         self.source_selector.setCurrentIndex(idx)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def path_format_finished(self):
         if self.source:
@@ -412,7 +414,7 @@ class TabWidget(QtWidgets.QWidget):
                 self.config_section, 'path_format', self.nm.format_string)
         self.show_file_list()
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def refresh(self):
         was_blocked = self.source_selector.blockSignals(True)
@@ -452,7 +454,7 @@ class TabWidget(QtWidgets.QWidget):
     def do_not_close(self):
         if not self.file_copier:
             return False
-        dialog = QtWidgets.QMessageBox()
+        dialog = QtWidgets.QMessageBox(parent=self)
         dialog.setWindowTitle(translate(
             'ImporterTab', 'Photini: import in progress'))
         dialog.setText(translate(
@@ -464,13 +466,15 @@ class TabWidget(QtWidgets.QWidget):
             QtWidgets.QMessageBox.Close | QtWidgets.QMessageBox.Cancel)
         dialog.setDefaultButton(QtWidgets.QMessageBox.Cancel)
         result = dialog.exec_()
+        if result == QtWidgets.QMessageBox.Close:
+            self.stop_copy()
         return result == QtWidgets.QMessageBox.Cancel
 
-    @QtCore.pyqtSlot(list)
+    @QtSlot(list)
     def new_selection(self, selection):
         pass
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def list_files(self):
         file_data = {}
@@ -491,7 +495,7 @@ class TabWidget(QtWidgets.QWidget):
         self.file_data = file_data
         self.sort_file_list()
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def sort_file_list(self):
         if eval(self.config_store.get('controls', 'sort_date', 'False')):
@@ -531,7 +535,7 @@ class TabWidget(QtWidgets.QWidget):
         self.file_list_widget.scrollToItem(
             first_active, QtWidgets.QAbstractItemView.PositionAtTop)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def selection_changed(self):
         count = len(self.file_list_widget.selectedItems())
@@ -541,12 +545,12 @@ class TabWidget(QtWidgets.QWidget):
             self.move_button.setEnabled(count > 0)
             self.copy_button.setEnabled(count > 0)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def select_all(self):
         self.select_files(datetime.min)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def select_new(self):
         since = datetime.min
@@ -580,12 +584,12 @@ class TabWidget(QtWidgets.QWidget):
         self.file_list_widget.scrollToItem(
             first_active, QtWidgets.QAbstractItemView.PositionAtTop)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def move_selected(self):
         self.copy_selected(move=True)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def copy_selected(self, move=False):
         copy_list = []
@@ -607,59 +611,52 @@ class TabWidget(QtWidgets.QWidget):
         else:
             self.copy_button.set_checked(True)
             self.move_button.setEnabled(False)
-        self.last_file_copied = None, datetime.min
+        last_file_copied = None, datetime.min
+        copier_result = deque()
         # start file copier in a separate thread
         self.file_copier = FileCopier(
-            self.source, copy_list, move, self.updating)
-        self.file_copier_thread = QtCore.QThread(self)
-        self.file_copier.moveToThread(self.file_copier_thread)
-        self.file_copier.output.connect(self.file_copied)
-        self.file_copier_thread.started.connect(self.file_copier.start)
-        self.file_copier_thread.start()
+            self.source, copy_list, move, copier_result)
+        copier_thread = QtCore.QThread(self)
+        self.file_copier.moveToThread(copier_thread)
+        copier_thread.started.connect(self.file_copier.start)
+        copier_thread.start()
+        # show files as they're copied
+        while self.file_copier.running:
+            if copier_result:
+                info, status = copier_result.popleft()
+                if not info:
+                    # copier thread has finished
+                    break
+                if status != 'ok':
+                    self._fail()
+                    break
+                if last_file_copied[1] < info['timestamp']:
+                    last_file_copied = info['dest_path'], info['timestamp']
+                for n in range(self.file_list_widget.count()):
+                    item = self.file_list_widget.item(n)
+                    if item.data(Qt.UserRole) == info['name']:
+                        item.setFlags(Qt.NoItemFlags)
+                        self.file_list_widget.scrollToItem(
+                            item, QtWidgets.QAbstractItemView.PositionAtTop)
+                        self.selection_changed()
+                        break
+                self.image_list.open_file(info['dest_path'])
+            else:
+                # wait for copier result
+                self.app.processEvents()
+        self.move_button.set_checked(False)
+        self.copy_button.set_checked(False)
+        self.file_copier = None
+        copier_thread.quit()
+        copier_thread.wait()
+        if last_file_copied[0]:
+            self.config_store.set(self.config_section, 'last_transfer',
+                                  last_file_copied[1].isoformat(' '))
+            self.image_list.done_opening(last_file_copied[0])
+        self.list_files()
 
-    @QtCore.pyqtSlot(dict, six.text_type)
-    @catch_all
-    def file_copied(self, info, status):
-        if not info:
-            # copier thread has finished
-            self.move_button.set_checked(False)
-            self.copy_button.set_checked(False)
-            self.file_copier = None
-            self.file_copier_thread.quit()
-            if self.last_file_copied[0]:
-                self.config_store.set(self.config_section, 'last_transfer',
-                                      self.last_file_copied[1].isoformat(' '))
-                self.image_list.done_opening(self.last_file_copied[0])
-            self.list_files()
-            return
-        if status != 'ok':
-            self._fail()
-            return
-        # block copier thread until image has been displayed
-        self.updating.lock()
-        self.image_list.open_file(info['dest_path'])
-        self.updating.unlock()
-        if self.last_file_copied[1] < info['timestamp']:
-            self.last_file_copied = info['dest_path'], info['timestamp']
-        for n in range(self.file_list_widget.count()):
-            item = self.file_list_widget.item(n)
-            if item.data(Qt.UserRole) == info['name']:
-                item.setFlags(Qt.NoItemFlags)
-                self.file_list_widget.scrollToItem(
-                    item, QtWidgets.QAbstractItemView.PositionAtTop)
-                self.selection_changed()
-                break
-
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def stop_copy(self):
         if self.file_copier:
             self.file_copier.running = False
-
-    @QtCore.pyqtSlot()
-    @catch_all
-    def shutdown(self):
-        if self.file_copier:
-            self.file_copier.running = False
-            self.file_copier_thread.quit()
-            self.file_copier_thread.wait()
