@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-19  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -29,9 +29,9 @@ import six
 from photini.imagelist import DRAG_MIMETYPE
 from photini.metadata import LatLon
 from photini.pyqt import (
-    catch_all, ComboBox, Qt, QtCore, QtGui, QtWebChannel,
-    QWebPage, QWebSettings, QWebView, QtWidgets, qt_version_info,
-    SingleLineEdit, using_qtwebengine)
+    catch_all, ComboBox, Qt, QtCore, QtGui, QtSignal, QtSlot, QtWebChannel,
+    QtWebEngineWidgets, QtWebKit, QtWebKitWidgets, QtWidgets, qt_version_info,
+    SingleLineEdit)
 from photini.technical import DoubleSpinBox
 
 logger = logging.getLogger(__name__)
@@ -54,6 +54,91 @@ class GeocoderBase(QtCore.QObject):
         self.block_timer.start()
 
 
+class CallHandler(QtCore.QObject):
+    # Simple object (with no attributes) for JavaScript to send signals
+    # to and hence invoke the methods JavaScript wants to call.
+    @QtSlot(int, six.text_type)
+    def log(self, level, message):
+        logger.log(level, message)
+
+    @QtSlot()
+    def initialize_finished(self):
+        self.parent().initialize_finished()
+
+    @QtSlot('QVariant')
+    def new_status(self, status):
+        self.parent().new_status(status)
+
+    @QtSlot(int)
+    def marker_click(self, marker_id):
+        self.parent().marker_click(marker_id)
+
+    @QtSlot(float, float)
+    def marker_drag(self, lat, lng):
+        self.parent().marker_drag(lat, lng)
+
+    @QtSlot(float, float, int)
+    def marker_drag_end(self, lat, lng, marker_id):
+        self.parent().marker_drag_end(lat, lng, marker_id)
+
+    @QtSlot(float, float)
+    def marker_drop(self, lat, lng):
+        self.parent().marker_drop(lat, lng)
+
+
+if QtWebEngineWidgets:
+    class QWebPage(QtWebEngineWidgets.QWebEnginePage):
+        def set_call_handler(self, call_handler):
+            self.web_channel = QtWebChannel.QWebChannel(parent=self)
+            self.setWebChannel(self.web_channel)
+            self.web_channel.registerObject('python', call_handler)
+
+        def acceptNavigationRequest(self, url, type_, isMainFrame):
+            if type_ == self.NavigationTypeLinkClicked:
+                if url.isLocalFile():
+                    url.setScheme('http')
+                QtGui.QDesktopServices.openUrl(url)
+                # delete temporary child created by createWindow
+                if isinstance(self.parent(), QWebPage):
+                    self.deleteLater()
+                return False
+            return super(QWebPage, self).acceptNavigationRequest(
+                url, type_, isMainFrame)
+
+        def createWindow(self, type_):
+            return QWebPage(parent=self)
+
+        def do_java_script(self, command):
+            self.runJavaScript(command)
+
+
+else:
+    class QWebPage(QtWebKitWidgets.QWebPage):
+        def __init__(self, call_handler, *args, **kwds):
+            super(QWebPage, self).__init__(*args, **kwds)
+            self.call_handler = call_handler
+            self.setLinkDelegationPolicy(self.DelegateAllLinks)
+            self.linkClicked.connect(self.link_clicked)
+            self.mainFrame().javaScriptWindowObjectCleared.connect(
+                self.java_script_window_object_cleared)
+
+        @QtSlot()
+        @catch_all
+        def java_script_window_object_cleared(self):
+            self.mainFrame().addToJavaScriptWindowObject(
+                'python', self.call_handler)
+
+        @QtSlot(QtCore.QUrl)
+        @catch_all
+        def link_clicked(self, url):
+            if url.isLocalFile():
+                url.setScheme('http')
+            QtGui.QDesktopServices.openUrl(url)
+
+        def do_java_script(self, command):
+            self.mainFrame().evaluateJavaScript(command)
+
+
 class MapWebPage(QWebPage):
     if qt_version_info >= (5, 6):
         def javaScriptConsoleMessage(self, level, msg, line, source):
@@ -64,106 +149,33 @@ class MapWebPage(QWebPage):
             logger.error('%s line %d: %s', source, line, msg)
 
 
-class MapWebEnginePage(MapWebPage):
-    def set_call_handler(self, call_handler):
-        self.web_channel = QtWebChannel.QWebChannel(parent=self)
-        self.setWebChannel(self.web_channel)
-        self.web_channel.registerObject('python', call_handler)
-
-    def acceptNavigationRequest(self, url, type_, isMainFrame):
-        if type_ == QWebPage.NavigationTypeLinkClicked:
-            if url.isLocalFile():
-                url.setScheme('http')
-            QtGui.QDesktopServices.openUrl(url)
-            # delete temporary child created by createWindow
-            if isinstance(self.parent(), MapWebEnginePage):
-                self.deleteLater()
-            return False
-        return super(MapWebEnginePage, self).acceptNavigationRequest(
-            url, type_, isMainFrame)
-
-    def createWindow(self, type_):
-        return MapWebEnginePage(parent=self)
-
-    def do_java_script(self, command):
-        self.runJavaScript(command)
+if QtWebEngineWidgets:
+    class QWebView(QtWebEngineWidgets.QWebEngineView):
+        def __init__(self, call_handler, *args, **kwds):
+            super(QWebView, self).__init__(*args, **kwds)
+            self.setPage(MapWebPage(parent=self))
+            self.page().set_call_handler(call_handler)
+            self.settings().setAttribute(
+                QtWebEngineWidgets.QWebEngineSettings.Accelerated2dCanvasEnabled, False)
+            self.settings().setAttribute(
+                QtWebEngineWidgets.QWebEngineSettings.LocalContentCanAccessRemoteUrls, True)
+            self.settings().setAttribute(
+                QtWebEngineWidgets.QWebEngineSettings.LocalContentCanAccessFileUrls, True)
 
 
-class MapWebKitPage(MapWebPage):
-    def __init__(self, call_handler, *args, **kwds):
-        super(MapWebKitPage, self).__init__(*args, **kwds)
-        self.call_handler = call_handler
-        self.setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
-        self.linkClicked.connect(self.link_clicked)
-        self.mainFrame().javaScriptWindowObjectCleared.connect(
-            self.java_script_window_object_cleared)
-
-    @QtCore.pyqtSlot()
-    @catch_all
-    def java_script_window_object_cleared(self):
-        self.mainFrame().addToJavaScriptWindowObject(
-            'python', self.call_handler)
-
-    @QtCore.pyqtSlot(QtCore.QUrl)
-    @catch_all
-    def link_clicked(self, url):
-        if url.isLocalFile():
-            url.setScheme('http')
-        QtGui.QDesktopServices.openUrl(url)
-
-    def do_java_script(self, command):
-        self.mainFrame().evaluateJavaScript(command)
-
-
-class CallHandler(QtCore.QObject):
-    # Simple object (with no attributes) for JavaScript to send signals
-    # to and hence invoke the methods JavaScript wants to call.
-    @QtCore.pyqtSlot(int, six.text_type)
-    def log(self, level, message):
-        logger.log(level, message)
-
-    @QtCore.pyqtSlot()
-    def initialize_finished(self):
-        self.parent().initialize_finished()
-
-    @QtCore.pyqtSlot(QtCore.QVariant)
-    def new_status(self, status):
-        self.parent().new_status(status)
-
-    @QtCore.pyqtSlot(int)
-    def marker_click(self, marker_id):
-        self.parent().marker_click(marker_id)
-
-    @QtCore.pyqtSlot(float, float)
-    def marker_drag(self, lat, lng):
-        self.parent().marker_drag(lat, lng)
-
-    @QtCore.pyqtSlot(float, float, int)
-    def marker_drag_end(self, lat, lng, marker_id):
-        self.parent().marker_drag_end(lat, lng, marker_id)
-
-    @QtCore.pyqtSlot(float, float)
-    def marker_drop(self, lat, lng):
-        self.parent().marker_drop(lat, lng)
+else:
+    class QWebView(QtWebKitWidgets.QWebView):
+        def __init__(self, call_handler, *args, **kwds):
+            super(QWebView, self).__init__(*args, **kwds)
+            self.setPage(MapWebPage(call_handler, parent=self))
+            self.settings().setAttribute(
+                QtWebKit.QWebSettings.LocalContentCanAccessRemoteUrls, True)
+            self.settings().setAttribute(
+                QtWebKit.QWebSettings.LocalContentCanAccessFileUrls, True)
 
 
 class MapWebView(QWebView):
-    drop_text = QtCore.pyqtSignal(int, int, six.text_type)
-
-    def __init__(self, call_handler, *args, **kwds):
-        super(MapWebView, self).__init__(*args, **kwds)
-        # set view's page
-        if using_qtwebengine:
-            self.setPage(MapWebEnginePage(parent=self))
-            self.page().set_call_handler(call_handler)
-            self.settings().setAttribute(
-                QWebSettings.Accelerated2dCanvasEnabled, False)
-        else:
-            self.setPage(MapWebKitPage(call_handler, parent=self))
-        self.settings().setAttribute(
-            QWebSettings.LocalContentCanAccessRemoteUrls, True)
-        self.settings().setAttribute(
-            QWebSettings.LocalContentCanAccessFileUrls, True)
+    drop_text = QtSignal(int, int, six.text_type)
 
     @catch_all
     def dragEnterEvent(self, event):
@@ -186,7 +198,7 @@ class MapWebView(QWebView):
 
 
 class LatLongDisplay(SingleLineEdit):
-    changed = QtCore.pyqtSignal()
+    changed = QtSignal()
 
     def __init__(self, image_list, *args, **kwds):
         super(LatLongDisplay, self).__init__(*args, **kwds)
@@ -197,7 +209,7 @@ class LatLongDisplay(SingleLineEdit):
         self.setEnabled(False)
         self.editingFinished.connect(self.editing_finished)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def editing_finished(self):
         text = self.get_value().strip()
@@ -304,7 +316,7 @@ class PhotiniMap(QtWidgets.QWidget):
         # other init
         self.image_list.image_list_changed.connect(self.image_list_changed)
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def image_list_changed(self):
         self.redraw_markers()
@@ -312,7 +324,7 @@ class PhotiniMap(QtWidgets.QWidget):
         self.update_altitude()
         self.see_selection()
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def initialise(self):
         page = '''<!DOCTYPE html>
@@ -334,7 +346,7 @@ class PhotiniMap(QtWidgets.QWidget):
 </html>'''
         lat, lng = eval(self.app.config_store.get('map', 'centre', '(51.0, 0.0)'))
         zoom = int(eval(self.app.config_store.get('map', 'zoom', '11')))
-        if using_qtwebengine:
+        if QtWebEngineWidgets:
             initialize = '''    <script type="text/javascript"
       src="qrc:///qtwebchannel/qwebchannel.js">
     </script>
@@ -390,7 +402,7 @@ class PhotiniMap(QtWidgets.QWidget):
                 self.app.config_store.set(
                     'map', key, repr(self.map_status[key]))
 
-    @QtCore.pyqtSlot(int, int, six.text_type)
+    @QtSlot(int, int, six.text_type)
     @catch_all
     def drop_text(self, x, y, text):
         self.dropped_images = eval(text)
@@ -406,14 +418,14 @@ class PhotiniMap(QtWidgets.QWidget):
         self.coords.refresh()
         self.see_selection()
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def new_coords(self):
         self.redraw_markers()
         self.update_altitude()
         self.see_selection()
 
-    @QtCore.pyqtSlot(object)
+    @QtSlot(object)
     @catch_all
     def new_altitude(self, value):
         for image in self.image_list.get_selected_images():
@@ -447,14 +459,14 @@ class PhotiniMap(QtWidgets.QWidget):
             latlong = image.metadata.latlong
             if not latlong:
                 continue
-            location = [latlong.lat, latlong.lon]
+            location = [latlong['lat'], latlong['lon']]
             if location not in locations:
                 locations.append(location)
         if not locations:
             return
         self.JavaScript('fitPoints({})'.format(repr(locations)))
 
-    @QtCore.pyqtSlot(list)
+    @QtSlot(list)
     @catch_all
     def new_selection(self, selection):
         self.redraw_markers()
@@ -486,7 +498,7 @@ class PhotiniMap(QtWidgets.QWidget):
                     'selected': image.selected,
                     }
                 self.JavaScript('addMarker({:d},{!r},{!r},{:d})'.format(
-                    marker_id, latlong.lat, latlong.lon, image.selected))
+                    marker_id, latlong['lat'], latlong['lon'], image.selected))
         for marker_id in list(self.marker_info.keys()):
             info = self.marker_info[marker_id]
             if not info['images']:
@@ -503,14 +515,14 @@ class PhotiniMap(QtWidgets.QWidget):
             latlngs.append([[x[1], x[2]] for x in t])
         self.JavaScript('plotTrack({!r})'.format(latlngs))
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def get_altitude(self):
         altitude = self.geocoder.get_altitude(self.coords.get_value())
         if altitude is not None:
             self.new_altitude(round(altitude, 1))
 
-    @QtCore.pyqtSlot()
+    @QtSlot()
     @catch_all
     def search(self, search_string=None, bounded=True):
         if not search_string:
@@ -539,7 +551,7 @@ class PhotiniMap(QtWidgets.QWidget):
             self.edit_box.addItem(
                 translate('MapTabsAll', '<repeat search>', 'repeat'))
 
-    @QtCore.pyqtSlot(int)
+    @QtSlot(int)
     @catch_all
     def goto_search_result(self, idx):
         self.edit_box.setCurrentIndex(0)
